@@ -1,9 +1,9 @@
-/***************************************************************************
-*   Copyright (C) 2005-2012 by Oleksandr Shneyder   *
-*   oleksandr.shneyder@obviously-nice.de   *
+/**************************************************************************
+*   Copyright (C) 2005-2012 by Oleksandr Shneyder                         *
+*   oleksandr.shneyder@obviously-nice.de                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  F*
+*   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
@@ -37,6 +37,8 @@ void x2goSession::operator = ( const x2goSession& s )
 
 bool	ONMainWindow::portable=false;
 QString ONMainWindow::homeDir;
+QString ONMainWindow::sessionCfg;
+
 
 #ifdef Q_OS_WIN
 QString ONMainWindow::u3Device;
@@ -57,6 +59,8 @@ ONMainWindow::ONMainWindow ( QWidget *parent ) :QMainWindow ( parent )
     extLogin=false;
     startMaximized=false;
     startHidden=false;
+    thinMode=false;
+    showHaltBtn=false;
     defaultUseSound=true;
     defaultSetKbd=true;
     defaultSetDPI=false;
@@ -64,6 +68,7 @@ ONMainWindow::ONMainWindow ( QWidget *parent ) :QMainWindow ( parent )
     extStarted=false;
     defaultLink=2;
     defaultFullscreen=false;
+    defaultXinerama=false;
     acceptRsa=false;
     cardStarted=false;
     cardReady=false;
@@ -96,6 +101,9 @@ ONMainWindow::ONMainWindow ( QWidget *parent ) :QMainWindow ( parent )
     lastSession=0l;
     changeBrokerPass=false;
 
+    appSeparator=0;
+
+
 #ifdef Q_OS_WIN
     clientSshPort="7022";
     pulsePort=4713;
@@ -161,16 +169,6 @@ ONMainWindow::ONMainWindow ( QWidget *parent ) :QMainWindow ( parent )
     widgetExtraStyle =new QPlastiqueStyle();
 #endif
 
-    QDesktopWidget wd;
-// 	x2goDebug<<"Desktop geometry:"<<wd.screenGeometry();
-    if ( wd.screenGeometry().width() <1024 ||
-            wd.screenGeometry().height() <768 )
-    {
-        miniMode=true;
-        x2goDebug<<"Switching to \"mini\" mode";
-    }
-
-
     agentCheckTimer=new QTimer ( this );
     connect ( agentCheckTimer,SIGNAL ( timeout() ),this,
               SLOT ( slotCheckAgentProcess() ) );
@@ -257,6 +255,19 @@ ONMainWindow::ONMainWindow ( QWidget *parent ) :QMainWindow ( parent )
         QTimer::singleShot ( 10, this, SLOT ( slotResize() ) );
     }
 
+
+    QDesktopWidget wd;
+
+    x2goDebug<<"primary screen geometry: "<<wd.screenGeometry(wd.screenNumber(this));
+
+    if ( wd.screenGeometry(wd.screenNumber(this)).width() <1024 ||
+            wd.screenGeometry(wd.screenNumber(this)).height() <768 )
+    {
+        miniMode=true;
+        x2goDebug<<"Switching to \"mini\" mode";
+    }
+
+
     if ( usePGPCard )
     {
         QTimer::singleShot ( 10, this, SLOT ( slotStartPGPAuth() ) );
@@ -392,6 +403,28 @@ ONMainWindow::ONMainWindow ( QWidget *parent ) :QMainWindow ( parent )
               SLOT ( slotResize ( const QSize ) ) );
     slotResize ( fr->size() );
 
+
+
+#ifdef Q_OS_LINUX
+    if (thinMode)
+    {
+        QTimer *timer = new QTimer(this);
+        connect(timer, SIGNAL(timeout()), this, SLOT(slotSyncX()));
+        timer->start(200);
+    }
+#endif
+    if (showHaltBtn)
+    {
+        QPushButton* bHalt=new QPushButton(bgFrame);
+        QPixmap p(":/png/power-button.png");
+        bHalt->setIcon(p);
+        bHalt->setFocusPolicy(Qt::NoFocus);
+        bHalt->setFixedSize(32,32);
+        bHalt->move(10,10);
+        bHalt->show();
+        connect(bHalt,SIGNAL(clicked()),this, SLOT(slotShutdownThinClient()));
+    }
+
     if (brokerMode)
     {
         broker=new HttpBrokerClient ( this, &config );
@@ -418,6 +451,12 @@ ONMainWindow::ONMainWindow ( QWidget *parent ) :QMainWindow ( parent )
     connect( xineramaTimer, SIGNAL(timeout()), this, SLOT(slotConfigXinerama()));
 
     x2goDebug<<"ONMainWindows constructor finished, home Directory is:"<<homeDir<<endl;
+    if (thinMode)
+    {
+        showMaximized();
+        activateWindow();
+        raise();
+    }
 }
 
 
@@ -429,8 +468,38 @@ ONMainWindow::~ONMainWindow()
     x2goDebug<<"end of ONMainWindow destructor";
 }
 
+void ONMainWindow::slotShutdownThinClient()
+{
+    QFile file(QDir::homePath()+"/.halt");
+    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
+        return;
+
+    QTextStream out(&file);
+    out << "\n";
+    file.close();
+}
 
 
+void ONMainWindow::slotSyncX()
+{
+    if (proxyWinId)
+    {
+        if (!isHidden())
+            hide();
+#ifdef Q_OS_LINUX
+        XSync(QX11Info::display(),false);
+#endif
+    }
+    else
+    {
+        if (isHidden())
+        {
+            showMaximized();
+            activateWindow();
+            raise();
+        }
+    }
+}
 
 
 void ONMainWindow::installTranslator()
@@ -927,7 +996,6 @@ void ONMainWindow::trayIconInit()
             delete trayIconMenu;
             trayIcon=0l;
             trayIconMenu=0l;
-
         }
     }
     else
@@ -937,13 +1005,39 @@ void ONMainWindow::trayIconInit()
             trayIconMenu = new QMenu(this);
             trayIconMenu->addAction(tr("Restore"),this, SLOT(showNormal()));
 
-
             trayIconActiveConnectionMenu = trayIconMenu->addMenu(tr("Not connected"));
 
+            appMenu[Application::MULTIMEDIA]=initTrayAppMenu(tr("Multimedia"),
+                                             QPixmap(":/icons/22x22/applications-multimedia.png"));
+            appMenu[Application::DEVELOPMENT]=initTrayAppMenu(tr("Development"),
+                                              QPixmap(":/icons/22x22/applications-development.png"));
+            appMenu[Application::EDUCATION]=initTrayAppMenu(tr("Education"),
+                                            QPixmap(":/icons/22x22/applications-education.png"));
+            appMenu[Application::GAME]=initTrayAppMenu(tr("Game"),
+                                       QPixmap(":/icons/22x22/applications-games.png"));
+            appMenu[Application::GRAPHICS]=initTrayAppMenu(tr("Graphics"),
+                                           QPixmap(":/icons/22x22/applications-graphics.png"));
+            appMenu[Application::NETWORK]=initTrayAppMenu(tr("Network"),
+                                          QPixmap(":/icons/22x22/applications-internet.png"));
+            appMenu[Application::OFFICE]=initTrayAppMenu(tr("Office"),
+                                         QPixmap(":/icons/22x22/applications-office.png"));
+            appMenu[Application::SETTINGS]=initTrayAppMenu(tr("Settings"),
+                                           QPixmap(":/icons/22x22/preferences-system.png"));
+            appMenu[Application::SYSTEM]=initTrayAppMenu(tr("System"),
+                                         QPixmap(":/icons/22x22/applications-system.png"));
+            appMenu[Application::UTILITY]=initTrayAppMenu(tr("Utility"),
+                                          QPixmap(":/icons/22x22/applications-utilities.png"));
+            appMenu[Application::OTHER]=initTrayAppMenu(tr("Other"),
+                                        QPixmap(":/icons/22x22/applications-other.png"));
+            appSeparator=trayIconActiveConnectionMenu->addSeparator();
+
 
             trayIconActiveConnectionMenu->addAction(tr ("Share folder..." ),this, SLOT(slotExportDirectory()));
             trayIconActiveConnectionMenu->addAction(tr("Suspend"),this, SLOT(slotSuspendSessFromSt()));
             trayIconActiveConnectionMenu->addAction(tr("Terminate"),this, SLOT(slotTermSessFromSt()));
+            connect (trayIconActiveConnectionMenu, SIGNAL(triggered(QAction*)), this, 
+		     SLOT(slotAppMenuTriggered(QAction*)));
+
 
             if (sessionStatusDlg && sessionStatusDlg->isVisible())
             {
@@ -953,7 +1047,9 @@ void ONMainWindow::trayIconInit()
                     trayIconActiveConnectionMenu->setTitle(lastUser->username());
             }
             else
+            {
                 trayIconActiveConnectionMenu->setEnabled(false);
+            }
             trayIconMenu->addSeparator();
             trayIconMenu->addAction(tr("Quit"),this, SLOT(trayQuit()));
 
@@ -969,10 +1065,76 @@ void ONMainWindow::trayIconInit()
             trayIcon->setToolTip(tr("Left mouse button to hide/restore - Right mouse button to display context menu"));
         }
         trayIcon->show();
+        plugAppsInTray();
     }
 #endif
 }
 
+QMenu* ONMainWindow::initTrayAppMenu(QString text, QPixmap icon)
+{
+    QMenu* menu=trayIconActiveConnectionMenu->addMenu(text);
+    menu->setIcon(icon);
+    return menu;
+}
+
+
+void ONMainWindow::slotAppMenuTriggered(QAction* action)
+{
+    x2goDebug<<"slotAppMenuTriggered :"<<action->data().toString()<<endl;
+    if(action->data().toString() != "")
+      runApplication(action->data().toString());
+}
+
+void ONMainWindow::plugAppsInTray()
+{
+    if (!trayIcon)
+        return;
+    removeAppsFromTray();
+    x2goDebug<<"plugging apps\n";
+    bool empty=true;
+    topActions.clear();
+    foreach(Application app, applications)
+    {
+      QAction* act;
+      if(app.category==Application::TOP)
+      {
+	 act=new QAction(app.icon,app.name,trayIconActiveConnectionMenu);
+         trayIconActiveConnectionMenu->insertAction(appSeparator, act);
+	 topActions.append(act);
+      }
+      else
+      {
+        act=appMenu[app.category]->addAction(app.icon,app.name);
+        appMenu[app.category]->menuAction()->setVisible(true);
+      }
+        act->setToolTip(app.comment);
+        act->setData(app.exec);
+        empty=false;
+    }
+    if (!empty)
+        appSeparator->setVisible(true);
+}
+
+
+void ONMainWindow::removeAppsFromTray()
+{
+    if (!trayIcon)
+        return;
+    x2goDebug<<"remove apps\n";
+    for (int i=0;i<=Application::OTHER;++i)
+    {
+        appMenu[i]->clear();
+        appMenu[i]->menuAction()->setVisible(false);
+    }
+    foreach (QAction* act, topActions)
+    {
+      trayIconActiveConnectionMenu->removeAction(act);
+      delete act;
+    }
+    topActions.clear();
+    appSeparator->setVisible(false);
+}
+
 
 QString ONMainWindow::findTheme ( QString /*theme*/ )
 {
@@ -1297,16 +1459,12 @@ void ONMainWindow::loadSettings()
 {
 
     X2goSettings st ( "sizes" );
-
     mwSize=st.setting()->value ( "mainwindow/size",
                                  ( QVariant ) QSize ( 800,600 ) ).toSize();
     mwPos=st.setting()->value ( "mainwindow/pos",
                                 ( QVariant ) QPoint ( 20,20 ) ).toPoint();
     mwMax=st.setting()->value ( "mainwindow/maximized",
                                 ( QVariant ) false ).toBool();
-    // tray stuff
-// 	trayQuitInfoShown = st1.value( "trayQuitInfoShown", false ).toBool();
-
 
 
     X2goSettings st1 ( "settings" );
@@ -2601,6 +2759,7 @@ void ONMainWindow::slotSshConnectionError ( QString message, QString lastSession
 {
     if ( sshConnection )
     {
+        sshConnection->wait();
         delete sshConnection;
         sshConnection=0l;
     }
