@@ -1,7 +1,7 @@
 
 /***************************************************************************
- *   Copyright (C) 2005-2012 by Oleksandr Shneyder   *
- *   oleksandr.shneyder@obviously-nice.de   *
+ *   Copyright (C) 2005-2012 by Oleksandr Shneyder                         *
+ *   oleksandr.shneyder@obviously-nice.de                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -55,6 +55,7 @@
 #if defined(CFGPLUGIN) && defined(Q_OS_LINUX)
 class QX11EmbedContainer;
 #endif
+class QToolButton;
 class QTemporaryFile;
 class QLineEdit;
 class QFrame;
@@ -118,6 +119,22 @@ struct serv
     QString sshPort;
 };
 
+struct Application
+{
+    QString name;
+    QString comment;
+    QString exec;
+    QPixmap icon;
+    enum {MULTIMEDIA, DEVELOPMENT, EDUCATION, GAME,
+          GRAPHICS, NETWORK, OFFICE,
+          SETTINGS, SYSTEM, UTILITY, OTHER, TOP
+         } category;
+    static bool lessThen(Application t1, Application t2)
+    {
+        return (t1.name.compare(t2.name,Qt::CaseInsensitive)<0);
+    }
+};
+
 struct x2goSession
 {
     QString agentPid;
@@ -131,7 +148,9 @@ struct x2goSession
     QString grPort;
     QString sndPort;
     QString fsPort;
+    bool published;
     int colorDepth;
+    bool fullscreen;
     enum {DESKTOP,ROOTLESS,SHADOW} sessionType;
     QString command;
     void operator = ( const x2goSession& s );
@@ -416,6 +435,16 @@ public:
     {
         return !noSessionEdit;
     }
+    const QList<Application>& getApplications()
+    {
+        return applications;
+    }
+    static QString getSessionConf()
+    {
+        return sessionCfg;
+    }
+
+    void runApplication(QString exec);
 
 
     SshMasterConnection* findServerSshConnection(QString host);
@@ -464,6 +493,7 @@ private:
     bool startMaximized;
     bool startHidden;
     bool defaultUseSound;
+    bool defaultXinerama;
     bool cardStarted;
     bool defaultSetKbd;
     bool showExport;
@@ -474,6 +504,7 @@ private:
     bool changeBrokerPass;
     bool connTest;
     bool embedMode;
+    bool thinMode;
     QString statusString;
     int defaultLink;
     int defaultQuality;
@@ -511,9 +542,9 @@ private:
     static QString homeDir;
     int retSessions;
     QList<serv> x2goServers;
+    QList<Application> applications;
 
     QPushButton* bSusp;
-    QPushButton* sbExp;
     QPushButton* bTerm;
     QPushButton* bNew;
     QPushButton* bShadow;
@@ -553,8 +584,10 @@ private:
     QString readExportsFrom;
     QString readLoginsFrom;
     QPushButton* sOk;
-    QPushButton* sbSusp;
-    QPushButton* sbTerm;
+    QToolButton* sbSusp;
+    QToolButton* sbExp;
+    QToolButton* sbTerm;
+    QToolButton* sbApps;
     QCheckBox* sbAdv;
     QPushButton* sCancel;
     QString resolution;
@@ -603,6 +636,7 @@ private:
     QAction *act_embedToolBar;
     QAction *act_changeBrokerPass;
     QAction *act_testCon;
+    QList <QAction*> topActions;
 
     QToolBar *stb;
 
@@ -615,6 +649,7 @@ private:
     QString nick;
     QString nfsPort;
     QString mntPort;
+    static QString sessionCfg;
     QProcess* ssh;
     QProcess* soundServer;
     QProcess* scDaemon;
@@ -626,6 +661,7 @@ private:
     bool proxyWinEmbedded;
     bool useLdap;
     bool showToolBar;
+    bool showHaltBtn;
     bool newSession;
     bool ldapOnly;
     bool isScDaemonOk;
@@ -742,6 +778,10 @@ private:
     QSystemTrayIcon *trayIcon;
     QMenu *trayIconMenu;
     QMenu *trayIconActiveConnectionMenu;
+
+    QAction* appSeparator;
+    QMenu* appMenu[Application::OTHER+1];
+
     bool trayEnabled;
     bool trayMinToTray;
     bool trayNoclose;
@@ -795,6 +835,10 @@ private:
             QString password, bool autologin, bool krbLogin, bool getSrv=false);
     void setProxyWinTitle();
     QRect proxyWinGeometry();
+    void readApplications();
+    void removeAppsFromTray();
+    void plugAppsInTray();
+    QMenu* initTrayAppMenu(QString text, QPixmap icon);
 
 
 protected:
@@ -811,8 +855,10 @@ private slots:
     void slotCheckXOrgConnection();
 #endif
 private slots:
+    void slotAppDialog();
     void slotShowPassForm();
     void displayUsers();
+    void slotAppMenuTriggered ( QAction * action );
     void slotPassChanged(const QString& result);
     void slotResize ( const QSize sz );
     void slotUnameChanged ( const QString& text );
@@ -820,7 +866,6 @@ private slots:
     void slotChangeBrokerPass();
     void slotTestConnection();
     void slotCheckPortableDir();
-
     void readUsers();
     void slotSelectedFromList ( UserButton* user );
     void slotUnameEntered();
@@ -835,6 +880,9 @@ private slots:
     void slotSshConnectionOk();
     void slotServSshConnectionOk(QString server);
     void slotChangeKbdLayout(const QString& layout);
+    void slotSyncX();
+    void slotShutdownThinClient();
+    void slotReadApplications(bool result, QString output, SshProcess* proc );
 
 public slots:
     void slotConfig();
