@@ -20,6 +20,7 @@
 
 #include "onmainwindow_privat.h"
 
+
 void ONMainWindow::slotSshConnectionOk()
 {
     x2goDebug<<"ssh connection ok"<<endl;
@@ -143,7 +144,7 @@ void ONMainWindow::slotSshUserAuthError ( QString error )
         sshConnection=0l;
     }
 
-    QMessageBox::critical ( 0l,tr ( "Authentification failed" ),error,
+    QMessageBox::critical ( 0l,tr ( "Authentication failed" ),error,
                             QMessageBox::Ok,
                             QMessageBox::NoButton );
 // 	currentKey=QString::null;
@@ -235,6 +236,8 @@ bool ONMainWindow::startSession ( const QString& sid )
     user=getCurrentUname();
     runRemoteCommand=true;
     shadowSession=false;
+    applications.clear();
+    removeAppsFromTray();
 
     if ( managedMode )
     {
@@ -276,8 +279,9 @@ bool ONMainWindow::startSession ( const QString& sid )
         user=st.setting()->value ( sid+"/user",
                                    ( QVariant ) QString::null ).toString();
         login->setText(user);
-        sshPort=st.setting()->value ( sid+"/sshport",
-                                      ( QVariant ) "22" ).toString();
+        sshPort=config.sshport;
+        /*        sshPort=st.setting()->value ( sid+"/sshport",
+                                              ( QVariant ) "22" ).toString();*/
     }
     if (sshConnection)
         sshConnection->disconnectSession();
@@ -446,6 +450,7 @@ void ONMainWindow::startNewSession()
     int speed;
     bool usekbd;
     bool rootless=false;
+    resumingSession.published=false;
     bool setDPI=defaultSetDPI;
     uint dpi=defaultDPI;
     QString layout;
@@ -454,7 +459,7 @@ void ONMainWindow::startNewSession()
     QString xdmcpServer;
     runRemoteCommand=true;
     QString host=QString::null;
-
+    removeAppsFromTray();
     if ( useLdap )
     {
         pack=defaultPack;
@@ -546,6 +551,8 @@ void ONMainWindow::startNewSession()
 
             rootless=st->setting()->value ( sid+"/rootless",
                                             ( QVariant ) false ).toBool();
+            resumingSession.published=st->setting()->value ( sid+"/published",
+                                      ( QVariant ) false ).toBool();
             xdmcpServer=st->setting()->value ( sid+"/xdmcpserver",
                                                ( QVariant )
                                                "localhost" ).toString();
@@ -707,6 +714,11 @@ void ONMainWindow::startNewSession()
         sessTypeStr="R ";
     if ( shadowSession )
         sessTypeStr="S ";
+    if ( resumingSession.published)
+    {
+        sessTypeStr="P ";
+        command="PUBLISHED";
+    }
     QString dpiEnv;
     QString xdmcpEnv;
     if ( runRemoteCommand==false && command=="XDMCP" )
@@ -730,6 +742,7 @@ void ONMainWindow::startNewSession()
         cmd+=sessTypeStr+QString::number ( shadowMode ) +"XSHAD"+
              shadowUser+"XSHAD"+shadowDisplay;
     }
+    resumingSession.fullscreen=fullscreen;
     x2goDebug<<cmd<<endl;
     if ( managedMode )
     {
@@ -756,7 +769,8 @@ void ONMainWindow::startNewSession()
 void ONMainWindow::resumeSession ( const x2goSession& s )
 {
     newSession=false;
-
+    applications.clear();
+    removeAppsFromTray();
     QString passwd=getCurrentPass();
     QString user=getCurrentUname();
     QString host=s.server;
@@ -771,6 +785,7 @@ void ONMainWindow::resumeSession ( const x2goSession& s )
     bool usekbd;
     QString layout;
     QString type;
+    removeAppsFromTray();
 
     if ( useLdap )
     {
@@ -908,6 +923,7 @@ void ONMainWindow::resumeSession ( const x2goSession& s )
         geometry=QString::number ( width ) +"x"+
                  QString::number ( height );
     }
+    resumingSession.fullscreen=fullscreen;
     QString link;
     switch ( speed )
     {
@@ -952,6 +968,15 @@ void ONMainWindow::resumeSession ( const x2goSession& s )
     type="query";
 #endif
 
+    if (s.sessionId.indexOf("RPUBLISHED")!=-1)
+    {
+        resumingSession.published=true;
+        sbApps->setDisabled(true);
+    }
+    else
+        resumingSession.published=false;
+
+
 
     if ( selectSessionDlg->isVisible() )
     {
@@ -1553,7 +1578,30 @@ void ONMainWindow::slotRetResumeSess ( bool result,
         }
     }
     else
+    {
         host=resumingSession.server;
+        QStringList outputLines=output.split("\n",QString::SkipEmptyParts);
+        foreach(QString line,outputLines)
+        {
+            if (line.indexOf("gr_port=")!=-1)
+            {
+                resumingSession.grPort=line.replace("gr_port=","");
+                x2goDebug<<"new gr_port: "<<resumingSession.grPort<<endl;
+            }
+            if (line.indexOf("sound_port=")!=-1)
+            {
+                resumingSession.sndPort=line.replace("sound_port=","");
+                x2goDebug<<"new sound_port: "<<resumingSession.sndPort<<endl;
+            }
+            if (line.indexOf("fs_port=")!=-1)
+            {
+                resumingSession.fsPort=line.replace("fs_port=","");
+                x2goDebug<<"new fs_port: "<<resumingSession.fsPort<<endl;
+            }
+        }
+        if (resumingSession.published)
+            readApplications();
+    }
     if ( !useLdap )
     {
         if ( !embedMode )
@@ -1877,12 +1925,17 @@ void ONMainWindow::slotTunnelOk()
     env << "NX_CLIENT="+QCoreApplication::applicationFilePath ();
 
 #if defined ( Q_OS_WIN ) || defined ( Q_OS_DARWIN )
+    // On Mac OS X, we want to make sure that DISPLAY is set to a proper value,
+    // but at the same time don't want to set the value ourselves but keep
+    // the provided one.
     QString disp=getXDisplay();
     if ( disp==QString::null )
     {
         //slotProxyerror ( QProcess::FailedToStart );
         return;
     }
+#endif // Q_OS_WIN || Q_OS_DARWIN
+#if defined ( Q_OS_WIN )
     if ( dispInd==-1 )
     {
         env <<"DISPLAY=localhost:"+disp;
@@ -1894,8 +1947,8 @@ void ONMainWindow::slotTunnelOk()
         /*		x2goDebug<<"existing env DISPLAY("<<dispInd<<
         		") DISPLAY=localhost:"+disp<<endl;*/
     }
-#endif
-#ifdef Q_OS_DARWIN
+#endif // Q_OS_WIN
+#if defined ( Q_OS_DARWIN )
     //setting /usr/X11/bin to find xauth
     env.insert (
         0,
@@ -2206,7 +2259,7 @@ void ONMainWindow::slotProxyStderr()
                 exportTimer->start ( 2000 );
             }
         }
-        sbSusp->setText ( tr ( "Suspend" ) );
+        sbSusp->setToolTip ( tr ( "Suspend" ) );
         if ( newSession )
         {
             runCommand();
@@ -2455,6 +2508,10 @@ void ONMainWindow::setStatStatus ( QString status )
 
         slVal->setFixedSize ( slVal->sizeHint() );
         sessionStatusDlg->show();
+        if (resumingSession.published)
+            sbApps->show();
+        else
+            sbApps->hide();
     }
     else
     {
