@@ -1,9 +1,9 @@
-/***************************************************************************
-*   Copyright (C) 2005-2012 by Oleksandr Shneyder   *
-*   oleksandr.shneyder@obviously-nice.de   *
+/**************************************************************************
+*   Copyright (C) 2005-2012 by Oleksandr Shneyder                         *
+*   oleksandr.shneyder@obviously-nice.de                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  F*
+*   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
@@ -20,7 +20,6 @@
 
 #include "onmainwindow_privat.h"
 
-
 x2goSession ONMainWindow::getNewSessionFromString ( const QString& string )
 {
     QStringList lst=string.split ( '|' );
@@ -37,6 +36,12 @@ x2goSession ONMainWindow::getNewSessionFromString ( const QString& string )
 }
 
 
+void ONMainWindow::slotAppDialog()
+{
+    AppDialog dlg(this);
+    dlg.exec();
+}
+
 void ONMainWindow::runCommand()
 {
     QString passwd=getCurrentPass();
@@ -49,13 +54,14 @@ void ONMainWindow::runCommand()
     QString rdpWidth;
     QString rdpHeight;
     bool rootless=false;
+    resumingSession.published=false;
     if ( !embedMode )
     {
         X2goSettings* st;
-	if(!brokerMode)
-	  st=new X2goSettings( "sessions" );
-	else
-	  st=new X2goSettings(config.iniFile, QSettings::IniFormat);
+        if (!brokerMode)
+            st=new X2goSettings( "sessions" );
+        else
+            st=new X2goSettings(config.iniFile, QSettings::IniFormat);
 
 
         if ( useLdap )
@@ -73,7 +79,9 @@ void ONMainWindow::runCommand()
                           sid+"/rdpserver",
                           ( QVariant ) "" ).toString();
             rootless=st->setting()->value ( sid+"/rootless",
-                                           ( QVariant ) false ).toBool();
+                                            ( QVariant ) false ).toBool();
+            resumingSession.published=st->setting()->value ( sid+"/published",
+                                      ( QVariant ) false ).toBool();
 
             rdpFS=st->setting()->value (
                       sid+"/fullscreen",
@@ -95,6 +103,11 @@ void ONMainWindow::runCommand()
     }
     if ( rootless )
         sessionType="R";
+    if ( resumingSession.published )
+    {
+        sessionType="P";
+        command="PUBLISHED";
+    }
 
     if ( command=="KDE" )
     {
@@ -186,6 +199,16 @@ void ONMainWindow::runCommand()
 #endif
 }
 
+
+void ONMainWindow::runApplication(QString exec)
+{
+    SshProcess* proc=new SshProcess ( sshConnection, this );
+    proc->startNormal ("PULSE_CLIENTCONFIG=~/.x2go/C-"+
+                       resumingSession.sessionId+"/.pulse-client.conf DISPLAY=:"+
+                       resumingSession.display+
+                       " setsid "+exec+">& /dev/null & exit");
+}
+
 void ONMainWindow::slotRetRunCommand ( bool result, QString output,
                                        SshProcess* proc )
 {
@@ -203,6 +226,150 @@ void ONMainWindow::slotRetRunCommand ( bool result, QString output,
                                 QMessageBox::Ok,
                                 QMessageBox::NoButton );
     }
+    else
+    {
+        if (resumingSession.published)
+            readApplications();
+    }
+}
+
+void ONMainWindow::readApplications()
+{
+    SshProcess* proc=new SshProcess ( sshConnection, this );
+    connect ( proc,SIGNAL ( sshFinished ( bool, QString,
+                                          SshProcess* ) ),
+              this,SLOT ( slotReadApplications ( bool,
+                                                 QString,
+                                                 SshProcess* ) ) );
+    proc->startNormal ( "x2gogetapps" );
+    sbApps->setEnabled(false);
+}
+
+void ONMainWindow::slotReadApplications(bool result, QString output,
+                                        SshProcess* proc )
+{
+    if ( proc )
+        delete proc;
+    if ( result==false )
+    {
+        QString message=tr ( "<b>Connection failed</b>\n:\n" ) +output;
+        if ( message.indexOf ( "publickey,password" ) !=-1 )
+        {
+            message=tr ( "<b>Wrong password!</b><br><br>" ) +
+                    message;
+        }
+        QMessageBox::critical ( 0l,tr ( "Error" ),message,
+                                QMessageBox::Ok,
+                                QMessageBox::NoButton );
+        return;
+    }
+    sbApps->setEnabled(true);
+    applications.clear();
+    QString locallong=QLocale::system().name();
+    QString localshort=QLocale::system().name().split("_")[0];
+
+    foreach(QString appstr, output.split("</desktop>",QString::SkipEmptyParts))
+    {
+        bool localcomment=false;
+        bool localname=false;
+        Application app;
+        app.category=Application::OTHER;
+        QStringList lines=appstr.split("\n", QString::SkipEmptyParts);
+        for (int i=0; i<lines.count(); ++i)
+        {
+            QString line=lines[i];
+            if (line.indexOf("Name["+localshort+"]=")!=-1  || line.indexOf("Name["+locallong+"]=")!=-1)
+            {
+                app.name=QString::fromUtf8(line.split("=")[1].toAscii());
+//                 x2goDebug<<"local name: "<<app.name<<endl;
+                localname=true;
+            }
+            if (line.indexOf("Comment["+localshort+"]=")!=-1 || line.indexOf("Comment["+locallong+"]=")!=-1)
+            {
+                app.comment=QString::fromUtf8(line.split("=")[1].toAscii());
+//                 x2goDebug<<"local comment: "<<app.comment<<endl;
+                localcomment=true;
+            }
+            if (line.indexOf("Name=")!=-1 && !localname)
+            {
+                app.name=line.split("=")[1];
+//                 x2goDebug<<"name: "<<app.name<<endl;
+            }
+            if (line.indexOf("Comment=")!=-1 && !localcomment)
+            {
+                app.comment=line.split("=")[1];
+//                 x2goDebug<<"comment: "<<app.comment<<endl;
+            }
+            if (line.indexOf("Exec=")!=-1)
+            {
+                app.exec=line.split("=")[1];
+                app.exec.replace("%f","",Qt::CaseInsensitive);
+                app.exec.replace("%u","",Qt::CaseInsensitive);
+//                 x2goDebug<<"exec: "<<app.exec<<endl;
+            }
+            if (line.indexOf("Categories=")!=-1)
+            {
+                if (line.indexOf("Audio")!=-1)
+                    app.category=Application::MULTIMEDIA;
+                if (line.indexOf("Vide")!=-1)
+                    app.category=Application::MULTIMEDIA;
+                if (line.indexOf("Development")!=-1)
+                    app.category=Application::DEVELOPMENT;
+                if (line.indexOf("Education")!=-1)
+                    app.category=Application::EDUCATION;
+                if (line.indexOf("Game")!=-1)
+                    app.category=Application::GAME;
+                if (line.indexOf("Graphics")!=-1)
+                    app.category=Application::GRAPHICS;
+                if (line.indexOf("Network")!=-1)
+                    app.category=Application::NETWORK;
+                if (line.indexOf("Office")!=-1)
+                    app.category=Application::OFFICE;
+                if (line.indexOf("Settings")!=-1)
+                    app.category=Application::SETTINGS;
+                if (line.indexOf("System")!=-1)
+                    app.category=Application::SYSTEM;
+                if (line.indexOf("Utility")!=-1)
+                    app.category=Application::UTILITY;
+                if (line.indexOf("X2Go-Top",0,Qt::CaseInsensitive)!=-1)
+                    app.category=Application::TOP;
+            }
+            if (line.indexOf("<icon>")!=-1)
+            {
+                bool isSvg=false;
+                line=lines[++i];
+                QByteArray pic;
+                while (line.indexOf("</icon>")==-1)
+                {
+                    pic+=QByteArray::fromBase64(line.toAscii());
+                    line=lines[++i];
+                    if (QString(QByteArray::fromBase64(line.toAscii())).indexOf("</svg>",Qt::CaseInsensitive)!=-1)
+                    {
+                        isSvg=true;
+                    }
+                }
+                if (!isSvg)
+                    app.icon.loadFromData(pic);
+                else
+                {
+                    QPixmap pix(32,32);
+                    QSvgRenderer svgRenderer( pic );
+                    QPainter pixPainter(&pix);
+                    svgRenderer.render(&pixPainter);
+                    app.icon=pix;
+                }
+            }
+        }
+        if (app.name.length()>0)
+        {
+            if (app.comment.length()<=0)
+                app.comment=app.name;
+            applications.append(app);
+        }
+    }
+
+    qSort(applications.begin(), applications.end(),Application::lessThen);
+    plugAppsInTray();
 }
 
 
@@ -230,10 +397,10 @@ bool ONMainWindow::parseParameter ( QString param )
         cleanAllFiles=true;
         return true;
     }
-    if(param == "--connectivity-test")
+    if (param == "--connectivity-test")
     {
-      connTest=true;
-      return true;
+        connTest=true;
+        return true;
     }
 
     if ( param=="--no-menu" )
@@ -247,6 +414,22 @@ bool ONMainWindow::parseParameter ( QString param )
         startMaximized=true;
         return true;
     }
+    if ( param=="--xinerama" )
+    {
+        defaultXinerama=true;
+        return true;
+    }
+    if (param == "--thinclient")
+    {
+        thinMode=true;
+        startMaximized=true;
+        return true;
+    }
+    if (param == "--haltbt")
+    {
+        showHaltBtn=true;
+        return true;
+    }
     if ( param=="--hide" )
     {
         startHidden=true;
@@ -272,12 +455,12 @@ bool ONMainWindow::parseParameter ( QString param )
         noSessionEdit=true;
         return true;
     }
-    if( param=="--change-broker-pass")
+    if ( param=="--change-broker-pass")
     {
-      changeBrokerPass=true;
-      return true;
+        changeBrokerPass=true;
+        return true;
     }
-    
+
 
     QString setting,value;
     QStringList vals=param.split ( "=" );
@@ -308,8 +491,8 @@ bool ONMainWindow::parseParameter ( QString param )
     if ( setting=="--kbd-layout" )
     {
         defaultLayout=value.split(",",QString::SkipEmptyParts);
-	if(defaultLayout.size()==0)
-	  defaultLayout<<tr("us");
+        if (defaultLayout.size()==0)
+            defaultLayout<<tr("us");
         return true;
     }
     if ( setting=="--session" )
@@ -318,6 +501,11 @@ bool ONMainWindow::parseParameter ( QString param )
         defaultSessionName=value;
         return true;
     }
+    if ( setting=="--session-conf" )
+    {
+        ONMainWindow::sessionCfg=value;
+        return true;
+    }
     if ( setting=="--sessionid" )
     {
         defaultSession=true;
@@ -389,84 +577,84 @@ bool ONMainWindow::parseParameter ( QString param )
         embedParent=value.toLong();
         return true;
     }
-    if( setting == "--broker-url")
+    if ( setting == "--broker-url")
     {
-      brokerMode=true;
-      noSessionEdit=true;
-      config.brokerurl=value;
-      return true;
+        brokerMode=true;
+        noSessionEdit=true;
+        config.brokerurl=value;
+        return true;
     }
-    if( setting == "--broker-name")
+    if ( setting == "--broker-name")
     {
-      config.brokerName=value;
-      return true;
+        config.brokerName=value;
+        return true;
     }
-    if( setting == "--auth-id")
+    if ( setting == "--auth-id")
     {
-      QFile file(value);
-     if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
-     {
-         printError ( param + tr(" (can't open file)"));
-         return false;
-     }
-         QTextStream in(&file);
-         config.brokerUserId = in.readLine();
-	 return true;
+        QFile file(value);
+        if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
+        {
+            printError ( param + tr(" (can't open file)"));
+            return false;
+        }
+        QTextStream in(&file);
+        config.brokerUserId = in.readLine();
+        return true;
     }
-    if(setting == "--support-menu")
+    if (setting == "--support-menu")
     {
-      if(! QFile::exists(value))
-      {
-	printError( param + tr(" (file not exists)"));
-	return false;
-      }
-      supportMenuFile=value;
-      return true;
+        if (! QFile::exists(value))
+        {
+            printError( param + tr(" (file not exists)"));
+            return false;
+        }
+        supportMenuFile=value;
+        return true;
     }
-    if(setting == "--background")
+    if (setting == "--background")
     {
-      if(! QFile::exists(value))
-      {
-	printError( param + tr(" (file not exists)"));
-	return false;
-      }
-      BGFile=value;
-      return true;
+        if (! QFile::exists(value))
+        {
+            printError( param + tr(" (file not exists)"));
+            return false;
+        }
+        BGFile=value;
+        return true;
     }
-    if(setting == "--session-icon")
+    if (setting == "--session-icon")
     {
-      if(! QFile::exists(value))
-      {
-	printError( param + tr(" (file not exists)"));
-	return false;
-      }
-      SPixFile=value;
-      return true;
+        if (! QFile::exists(value))
+        {
+            printError( param + tr(" (file not exists)"));
+            return false;
+        }
+        SPixFile=value;
+        return true;
     }
-    if(setting == "--home")
+    if (setting == "--home")
     {
-      QDir dr;
-      
+        QDir dr;
+
 #ifdef Q_OS_WIN
-      int find=value.indexOf("(");
-      int lind=value.indexOf(")");
-      if(find!=-1 && lind !=-1)
-      {	
-	QString label=value.mid(find+1,lind-find-1);
-	x2goDebug<< "searching for drive with label: "<<label;
-	QString drive=wapiGetDriveByLabel(label);
-	value.replace("("+label+")",drive);
-	x2goDebug<<"new path: "<<value;
-      }
+        int find=value.indexOf("(");
+        int lind=value.indexOf(")");
+        if (find!=-1 && lind !=-1)
+        {
+            QString label=value.mid(find+1,lind-find-1);
+            x2goDebug<< "searching for drive with label: "<<label;
+            QString drive=wapiGetDriveByLabel(label);
+            value.replace("("+label+")",drive);
+            x2goDebug<<"new path: "<<value;
+        }
 #endif
-      if(! dr.exists(value))
-      {
-	printError( param + tr(" (directory not exists)"));
-	return false;
-      }
-      homeDir=value;
-      portableDataPath=value;
-      return true;
+        if (! dr.exists(value))
+        {
+            printError( param + tr(" (directory not exists)"));
+            return false;
+        }
+        homeDir=value;
+        portableDataPath=value;
+        return true;
     }
 
     printError ( param );
@@ -689,12 +877,15 @@ void ONMainWindow::showHelp()
         "--help-pack\t\t\t show available pack methods\n"
         "--no-menu\t\t\t hide menu bar\n"
         "--no-session-edit\t\t not allow user to edit preconfigured\n"
-        "sessions\n"
+        "\t\t\t\t sessions\n"
         "--maximize\t\t\t start maximized\n"
         "--hide\t\t\t\t start hidden\n"
         "--portable\t\t\t start in \"portable\" mode\n"
         "--pgp-card\t\t\t use openPGP card authentication\n"
+        "--xinerama\t\t\t use Xinerama by default\n"
         "--ldap-printing\t\t\t allow client side printing in LDAP mode\n"
+        "--thinclient\t\t\t run without window manager\n"
+        "--haltbt\t\t\t show shutdown button\n"
         "--add-to-known-hosts\t\t add RSA key fingerprint to "
         ".ssh/known_hosts\n"
         "\t\t\t\t if authenticity of server can't be established\n\n"
@@ -716,10 +907,11 @@ void ONMainWindow::showHelp()
         "--pack=<packmethod>\t\t set default pack method, default "
         "'16m-jpeg-9'\n"
         "--kbd-layout=<layout>\t\t set default keyboard layout or layouts\n"
-        "comma separated\n"
+        "\t\t\t\t comma separated\n"
         "--kbd-type=<typed>\t\t set default keyboard type\n"
-        "--home=<dir>\t\t set users home directory\n"
-        "--set-kbd=<0|1>\t\t\t overwrite current keyboard settings\n" ;
+        "--home=<dir>\t\t\t set users home directory\n"
+        "--set-kbd=<0|1>\t\t\t overwrite current keyboard settings\n"
+        "--session-conf=<file>\t\t\t path to alternative session config\n";
     qCritical ( "%s",helpMsg.toLocal8Bit().data() );
     QMessageBox::information ( this,tr ( "Options" ),helpMsg );
 }
@@ -810,22 +1002,22 @@ void ONMainWindow::slotGetServers ( bool result, QString output,
 
     listedSessions.clear();
     retSessions=0;
-    if(sshConnection)
-      sshConnection->disconnectSession();
+    if (sshConnection)
+        sshConnection->disconnectSession();
     QString passwd;
     QString user=getCurrentUname();
     passwd=getCurrentPass();
-    for(int i=0; i< serverSshConnections.count();++i)
+    for (int i=0; i< serverSshConnections.count();++i)
     {
-      if(serverSshConnections[i])
-	serverSshConnections[i]->disconnectSession();
+        if (serverSshConnections[i])
+            serverSshConnections[i]->disconnectSession();
     }
     serverSshConnections.clear();
     for ( int j=0;j<x2goServers.size();++j )
     {
         QString host=x2goServers[j].name;
-	sshPort=x2goServers[j].sshPort;
-	serverSshConnections<<startSshConnection ( host,sshPort,acceptRsa,user,passwd,true,false,true);
+        sshPort=x2goServers[j].sshPort;
+        serverSshConnections<<startSshConnection ( host,sshPort,acceptRsa,user,passwd,true,false,true);
     }
 }
 
@@ -841,7 +1033,7 @@ void ONMainWindow::slotListAllSessions ( bool result,QString output,
     if ( proc )
         delete proc;
     proc=0;
-    
+
     if ( result==false )
     {
         QString message=tr ( "<b>Connection failed</b>\n" ) +output;
@@ -923,10 +1115,10 @@ void ONMainWindow::slotExportDirectory()
         return;
 
     bool hide_after=false;
-    if(isHidden())
+    if (isHidden())
     {
-      showNormal();
-      hide_after=true;
+        showNormal();
+        hide_after=true;
     }
     QString path;
     if ( !useLdap && !embedMode )
@@ -936,12 +1128,12 @@ void ONMainWindow::slotExportDirectory()
             path=dlg.getExport();
     }
     else
-           
+
         path= QFileDialog::getExistingDirectory (
                   this,QString::null,
                   homeDir );
-     if(hide_after)
-       hide();
+    if (hide_after)
+        hide();
 #ifdef Q_OS_WIN
     if ( ONMainWindow::getPortable() &&
             ONMainWindow::U3DevicePath().length() >0 )
@@ -1505,22 +1697,22 @@ void ONMainWindow::slotAboutQt()
 void ONMainWindow::slotSupport()
 {
     QFile file(supportMenuFile);
-     if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
-         return;
+    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
+        return;
 
-     QTextStream in(&file);
-     QString sup;
-     while (!in.atEnd()) 
-     {
-         sup+=in.readLine();
-     }     
-     QMessageBox::information (this,tr ( "Support" ),sup);
+    QTextStream in(&file);
+    QString sup;
+    while (!in.atEnd())
+    {
+        sup+=in.readLine();
+    }
+    QMessageBox::information (this,tr ( "Support" ),sup);
 }
 
 void ONMainWindow::slotAbout()
 {
     QString aboutStr=tr (
-                         "</b><br> (C. 2006-2011 <b>obviously nice</b>: "
+                         "</b><br> (C. 2006-2012 <b>obviously nice</b>: "
                          "Oleksandr Shneyder, Heinz-Markus Graesing)<br>" );
     if ( embedMode )
         aboutStr+=tr ( "<br>x2goplugin mode was sponsored by "
@@ -1948,7 +2140,7 @@ void ONMainWindow::slotGpgAgentFinished ( int , QProcess::ExitStatus )
 
     if ( !useLdap )
     {
-        if ( passForm->isVisible() )
+        if ( passForm->isVisible() && !brokerMode)
         {
             if ( passForm->isEnabled() )
             {
@@ -2006,11 +2198,11 @@ void ONMainWindow::slotCheckAgentProcess()
         cardStarted=false;
         if ( nxproxy )
             if ( nxproxy->state() ==QProcess::Running )
-	    {
-	          x2goDebug<<"Suspending session\n";
-	          slotSuspendSessFromSt();
+            {
+                x2goDebug<<"Suspending session\n";
+                slotSuspendSessFromSt();
 //                 nxproxy->terminate();
-	    }
+            }
     }
 
     x2goDebug<<"gpg-agent finished\n";
@@ -2035,63 +2227,29 @@ bool ONMainWindow::checkAgentProcess()
 #if defined ( Q_OS_DARWIN )
 QString ONMainWindow::getXDisplay()
 {
-    QTcpSocket tcpSocket ( this );
-    uint dispNumber=0;
-    QString xname,xdir,xopt;
-    dispNumber=0;
-    xdir=ConfigDialog::getXDarwinDirectory();
-    xname=xdir+"/Contents/MacOS/X11";
-    xopt=" -rootless :0";
+    QLocalSocket unixSocket (this);
+    QString xsocket (getenv ("DISPLAY"));
 
-    //for newer versions of XQuartz start startx instead of X11.app
-    xname="/usr/X11/bin/startx";
-    xopt="";
-    tcpSocket.connectToHost ( "127.0.0.1",6000+dispNumber );
+    // OS X starts the X11 server automatically, as soon as the launchd UNIX socket
+    // is accessed.
+    // On user login, the DISPLAY environment variable is set to this said existing
+    // socket.
+    // By now, we should have a socket. Test, if connecting works.
+    if ((!xsocket.isEmpty ()))
+    {
+        unixSocket.connectToServer (xsocket);
 
-    if ( tcpSocket.waitForConnected ( 3000 ) )
-    {
-        tcpSocket.close();
-        return QString::number ( dispNumber );
-    }
-    if ( xname==QString::null )
-    {
-        QMessageBox::critical (
-            this,tr ( "Can't connect to X-Server" ),
-            tr (
-                "Can't connect to X-Server\nPlease check your settings"
-            ) );
-        slotConfig();
-        return QString::null;
-    }
-    QProcess* pr=new QProcess ( this );
-    pr->setWorkingDirectory ( xdir );
-    pr->start ( xname+" "+xopt,QIODevice::NotOpen );
-    if ( pr->waitForStarted ( 3000 ) )
-    {
-#ifdef Q_OS_DARWIN
-        //FIXME: the call of unistd.h sleep() do not work on all
-        // Mac OS X systems
-        system ( "sleep 3" );
-#endif
-
-        tcpSocket.connectToHost ( "127.0.0.1",6000+dispNumber );
-        if ( tcpSocket.waitForConnected ( 1000 ) )
+        if (unixSocket.waitForConnected (10000))
         {
-            tcpSocket.close();
-            return QString::number ( dispNumber );
+            unixSocket.disconnectFromServer ();
+            return (xsocket);
         }
-        QMessageBox::critical (
-            this,tr ( "Can't connect to X-Server" ),
-            tr (
-                "Can't connect to X-Server\nPlease check your settings"
-            ) );
-        slotConfig();
-        return QString::null;
     }
+    // And if not, error out.
     QMessageBox::critical (
-        this,QString::null,
+        this,tr ( "Can't connect to X server\nPlease check your settings" ),
         tr (
-            "Can't start X Server\nPlease check your settings" ) );
+            "Can't start X server\nPlease check your settings" ) );
     slotConfig();
     return QString::null;
 }
