@@ -1,9 +1,9 @@
-/***************************************************************************
-*   Copyright (C) 2005-2012 by Oleksandr Shneyder   *
-*   oleksandr.shneyder@obviously-nice.de   *
+/**************************************************************************
+*   Copyright (C) 2005-2012 by Oleksandr Shneyder                         *
+*   oleksandr.shneyder@obviously-nice.de                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  F*
+*   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
@@ -1107,18 +1107,22 @@ void ONMainWindow::slotConfigXinerama()
     if (newGeometry==lastDisplayGeometry)
         return;
     lastDisplayGeometry=newGeometry;
-   x2goDebug<<"New proxy geometry: "<<lastDisplayGeometry<<endl;
+    x2goDebug<<"New proxy geometry: "<<lastDisplayGeometry<<endl;
     QDesktopWidget* root=QApplication::desktop();
     QList<QRect> newXineramaScreens;
     for (int i=0; i< root->numScreens();++i)
     {
-        QRect intersection=root->screenGeometry(i).intersected(lastDisplayGeometry);
+        QRect intersection;
+        if (resumingSession.fullscreen)
+            intersection=root->screenGeometry(i);
+        else
+            intersection=root->screenGeometry(i).intersected(lastDisplayGeometry);
         if (!intersection.isNull())
         {
-       x2goDebug<<"intersected with "<<i<<": "<<intersection<<endl;
+            x2goDebug<<"intersected with "<<i<<": "<<intersection<<endl;
             intersection.moveLeft(intersection.x()-lastDisplayGeometry.x());
             intersection.moveTop(intersection.y()-lastDisplayGeometry.y());
-       x2goDebug<<"xinerama screen: "<<intersection<<endl;
+            x2goDebug<<"xinerama screen: "<<intersection<<endl;
             newXineramaScreens<<intersection;
         }
     }
@@ -1143,6 +1147,8 @@ void ONMainWindow::slotConfigXinerama()
 
 void ONMainWindow::slotXineramaConfigured()
 {
+    if (resumingSession.fullscreen)
+        return;
     if (xinSizeInc == -1)
         xinSizeInc=1;
     else
@@ -1167,8 +1173,8 @@ void ONMainWindow::slotFindProxyWin()
 {
 #ifndef Q_OS_DARWIN
     x2goDebug<<"search proxy win: "<<"X2GO-"+resumingSession.sessionId;
-
     proxyWinId=findWindow ( "X2GO-"+resumingSession.sessionId );
+    bool xinerama=defaultXinerama;
     if ( proxyWinId )
     {
         x2goDebug<<"proxy win found:"<<proxyWinId;
@@ -1188,14 +1194,8 @@ void ONMainWindow::slotFindProxyWin()
             else
                 st= new X2goSettings( "sessions" );
             uint displays=QApplication::desktop()->numScreens();
-            if (st->setting()->value ( sid+"/xinerama",
-                                       ( QVariant ) false ).toBool())
-            {
-                x2goDebug<<"Starting xinerama timer\n";
-                lastDisplayGeometry=QRect();
-                xineramaScreens.clear();
-                xineramaTimer->start(500);
-            }
+            xinerama=st->setting()->value ( sid+"/xinerama",
+                                            ( QVariant ) defaultXinerama ).toBool();
             if (st->setting()->value ( sid+"/multidisp",
                                        ( QVariant ) false ).toBool())
             {
@@ -1209,6 +1209,14 @@ void ONMainWindow::slotFindProxyWin()
                 return;
             }
         }
+        if (xinerama)
+        {
+            x2goDebug<<"Starting xinerama timer\n";
+            lastDisplayGeometry=QRect();
+            xineramaScreens.clear();
+            xineramaTimer->start(500);
+        }
+
         if ( embedMode )
         {
             if ( config.rootless )
@@ -1632,12 +1640,19 @@ void ONMainWindow::initPassDlg()
         passForm->setFixedSize ( 310,180 );
     QPalette pal=passForm->palette();
     pal.setBrush ( QPalette::Window, QColor ( 255,255,255,0 ) );
+    pal.setColor ( QPalette::Active, QPalette::WindowText, QPalette::Mid );
+    pal.setColor ( QPalette::Active, QPalette::ButtonText, QPalette::Mid );
+    pal.setColor ( QPalette::Active, QPalette::Text, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::WindowText, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::ButtonText, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::Text, QPalette::Mid );
     passForm->setPalette ( pal );
 
     pal.setColor ( QPalette::Button, QColor ( 255,255,255,0 ) );
     pal.setColor ( QPalette::Window, QColor ( 255,255,255,255 ) );
     pal.setColor ( QPalette::Base, QColor ( 255,255,255,255 ) );
 
+
     QFont fnt=passForm->font();
     if ( miniMode )
 #ifdef Q_WS_HILDON
@@ -1705,9 +1720,6 @@ void ONMainWindow::initPassDlg()
 
 
 
-    pal.setColor ( QPalette::Button, QColor ( 255,255,255,0 ) );
-    pal.setColor ( QPalette::Window, QColor ( 255,255,255,255 ) );
-    pal.setColor ( QPalette::Base, QColor ( 255,255,255,255 ) );
     cbLayout->setPalette ( pal );
     ok->setPalette ( pal );
     cancel->setPalette ( pal );
@@ -1825,6 +1837,13 @@ void ONMainWindow::initStatusDlg()
     username->addWidget ( sessionStatusDlg );
     QPalette pal=sessionStatusDlg->palette();
     pal.setBrush ( QPalette::Window, QColor ( 0,0,0,0 ) );
+    pal.setColor ( QPalette::Active, QPalette::WindowText, QPalette::Mid );
+    pal.setColor ( QPalette::Active, QPalette::ButtonText, QPalette::Mid );
+    pal.setColor ( QPalette::Active, QPalette::Text, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::WindowText, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::ButtonText, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::Text, QPalette::Mid );
+
     sessionStatusDlg->setPalette ( pal );
 
     slName=new QLabel ( sessionStatusDlg );
@@ -1839,39 +1858,57 @@ void ONMainWindow::initStatusDlg()
 
     slVal->hide();
     slVal->setFixedHeight ( slName->sizeHint().height() );
-    sbSusp=new QPushButton ( tr ( "Abort" ),sessionStatusDlg );
-    sbTerm=new QPushButton ( tr ( "Terminate" ),sessionStatusDlg );
-    sbExp=new QPushButton ( tr ( "Share folder..." ),
-                            sessionStatusDlg );
+
+    sbApps=new QToolButton (sessionStatusDlg );
+    sbApps->setToolTip(tr ( "Applications..." ));
+    sbApps->setIcon(QPixmap(":/icons/32x32/apps.png"));
+    sbApps->setAutoRaise(true);
+    sbApps->setFocusPolicy(Qt::NoFocus);
+
+    sbExp=new QToolButton (sessionStatusDlg );
+    sbExp->setIcon(QPixmap(":/icons/32x32/open_dir.png"));
+    sbExp->setToolTip (tr ("Share folder..." ));
+    sbExp->setAutoRaise(true);
+    sbExp->setFocusPolicy(Qt::NoFocus);
+
+    sbSusp=new QToolButton (sessionStatusDlg );
+    sbSusp->setIcon(QPixmap(":/icons/32x32/suspend_session.png"));
+    sbSusp->setToolTip(tr ( "Abort" ));
+    sbSusp->setAutoRaise(true);
+    sbSusp->setFocusPolicy(Qt::NoFocus);
+
+
+    sbTerm=new QToolButton (sessionStatusDlg );
+    sbTerm->setIcon(QPixmap(":/icons/32x32/stop_session.png"));
+    sbTerm->setToolTip(tr ( "Terminate" ));
+    sbTerm->setAutoRaise(true);
+    sbTerm->setFocusPolicy(Qt::NoFocus);
+
+
     sbAdv=new QCheckBox ( tr ( "Show details" ),sessionStatusDlg );
     setWidgetStyle ( sbTerm );
+    setWidgetStyle ( sbApps );
     setWidgetStyle ( sbExp );
     setWidgetStyle ( sbSusp );
     setWidgetStyle ( sbAdv );
 
     sbAdv->setFixedSize ( sbAdv->sizeHint() );
-#ifndef Q_WS_HILDON
-    sbSusp->setFixedSize ( sbSusp->sizeHint() );
-    sbTerm->setFixedSize ( sbTerm->sizeHint() );
-    sbExp->setFixedSize ( sbExp->sizeHint() );
-#else
-    QSize sz=sbSusp->sizeHint();
-    sz.setWidth ( ( int ) ( sz.width() /1.5 ) );
-    sz.setHeight ( ( int ) ( sz.height() /1.5 ) );
-    sbSusp->setFixedSize ( sz );
-    sz=sbExp->sizeHint();
-    sz.setWidth ( ( int ) ( sz.width() ) );
-    sz.setHeight ( ( int ) ( sz.height() /1.5 ) );
-    sbExp->setFixedSize ( sz );
-    sz=sbTerm->sizeHint();
-    sz.setWidth ( ( int ) ( sz.width() /1.5 ) );
-    sz.setHeight ( ( int ) ( sz.height() /1.5 ) );
-    sbTerm->setFixedSize ( sz );
-#endif
+    sbApps->setFixedSize ( 32,32 );
+    sbSusp->setFixedSize ( 32,32 );
+    sbTerm->setFixedSize ( 32,32 );
+    sbExp->setFixedSize ( 32,32 );
+
+    /*
+        sbApps->setFocusPolicy(Qt::NoFocus);
+        sbSusp->setFocusPolicy(Qt::NoFocus);
+        sbTerm->setFocusPolicy(Qt::NoFocus);
+        sbExp->setFocusPolicy(Qt::NoFocus);*/
+
     sbAdv->hide();
     sbSusp->hide();
     sbTerm->hide();
     sbExp->hide();
+    sbApps->hide();
 
 
     pal.setColor ( QPalette::Button, QColor ( 255,255,255,0 ) );
@@ -1879,6 +1916,7 @@ void ONMainWindow::initStatusDlg()
     pal.setColor ( QPalette::Base, QColor ( 255,255,255,255 ) );
 
     sbAdv->setPalette ( pal );
+    sbApps->setPalette ( pal );
     sbSusp->setPalette ( pal );
     sbTerm->setPalette ( pal );
     sbExp->setPalette ( pal );
@@ -1901,6 +1939,8 @@ void ONMainWindow::initStatusDlg()
               SLOT ( slotShowAdvancedStat() ) );
     connect ( sbExp,SIGNAL ( clicked() ),this,
               SLOT ( slotExportDirectory() ) );
+    connect ( sbApps,SIGNAL ( clicked() ),this,
+              SLOT ( slotAppDialog()) );
 
     QVBoxLayout* layout=new QVBoxLayout ( sessionStatusDlg );
     QHBoxLayout* ll=new QHBoxLayout();
@@ -1915,10 +1955,11 @@ void ONMainWindow::initStatusDlg()
 
     QHBoxLayout* bl=new QHBoxLayout();
     bl->addStretch();
+    bl->addWidget ( sbApps );
     bl->addWidget ( sbExp );
     bl->addWidget ( sbSusp );
     bl->addWidget ( sbTerm );
-    bl->addStretch();
+//     bl->addStretch();
     layout->addLayout ( ll );
     layout->addStretch();
     layout->addWidget ( stInfo );
@@ -1969,6 +2010,13 @@ void ONMainWindow::initSelectSessDlg()
         selectSessionDlg->setFixedSize ( 310,180 );
     QPalette pal=selectSessionDlg->palette();
     pal.setBrush ( QPalette::Window, QColor ( 255,255,255,0 ) );
+    pal.setColor ( QPalette::Active, QPalette::WindowText, QPalette::Mid );
+    pal.setColor ( QPalette::Active, QPalette::ButtonText, QPalette::Mid );
+    pal.setColor ( QPalette::Active, QPalette::Text, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::WindowText, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::ButtonText, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::Text, QPalette::Mid );
+
     selectSessionDlg->setPalette ( pal );
 
     pal.setColor ( QPalette::Button, QColor ( 255,255,255,0 ) );
@@ -2478,18 +2526,13 @@ void ONMainWindow::slotGetBrokerSession(const QString& sinfo)
     int keyStartPos=sinfo.indexOf("-----BEGIN DSA PRIVATE KEY-----");
     QString endStr="-----END DSA PRIVATE KEY-----";
     int keyEndPos=sinfo.indexOf(endStr);
-    if (keyEndPos == -1 || keyStartPos == -1 || lst.size()==0)
-    {
-        //throw error
-        QMessageBox::critical (
-            0,tr ( "Error" ),
-            tr ("Invalid reply from broker") +"<br>"+sinfo);
-
-        close();
-        return;
-    }
-    config.server=(lst[1].split("\n"))[0];
-    config.key=sinfo.mid(keyStartPos, keyEndPos+endStr.length()-keyStartPos);
+    if (! (keyEndPos == -1 || keyStartPos == -1 || lst.size()==0))
+        config.key=sinfo.mid(keyStartPos, keyEndPos+endStr.length()-keyStartPos);
+    QString serverLine=(lst[1].split("\n"))[0];
+    QStringList words=serverLine.split(":",QString::SkipEmptyParts);
+    config.server=words[0];
+    if (words.count()>1)
+        config.sshport=words[1];
 //    x2goDebug<<"server: "<<config.server<<endl<<" key: "<<config.key;
     if (sinfo.indexOf("SESSION_INFO")!=-1)
     {
