@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2005-2012 by Oleksandr Shneyder   *
- *   oleksandr.shneyder@obviously-nice.de   *
+ *   Copyright (C) 2005-2012 by Oleksandr Shneyder                         *
+ *   oleksandr.shneyder@obviously-nice.de                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -23,6 +23,8 @@
 
 #include <QTextStream>
 #include <QTranslator>
+#include <QToolButton>
+#include <QPainter>
 
 #include "version.h"
 #include "x2goclientconfig.h"
@@ -30,6 +32,7 @@
 #include "userbutton.h"
 #include "exportdialog.h"
 #include "printprocess.h"
+#include "appdialog.h"
 #include <QDesktopServices>
 #include <QApplication>
 #include <QScrollBar>
@@ -66,7 +69,7 @@
 #include <QFileDialog>
 #include <QHttp>
 #include <QUrl>
-#include <QTcpSocket>
+#include <QLocalSocket>
 #include <QPlastiqueStyle>
 #include "sshprocess.h"
 #include "imgframe.h"
