@@ -33,6 +33,19 @@ SessionButton::SessionButton ( ONMainWindow* mw,QWidget *parent, QString id )
 {
     editable=mw->sessionEditEnabled();
 
+
+    QPalette pal=palette();
+    pal.setColor ( QPalette::Active, QPalette::WindowText, QPalette::Mid );
+    pal.setColor ( QPalette::Active, QPalette::ButtonText, QPalette::Mid );
+    pal.setColor ( QPalette::Active, QPalette::Text, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::WindowText, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::ButtonText, QPalette::Mid );
+    pal.setColor ( QPalette::Inactive, QPalette::Text, QPalette::Mid );
+
+    setPalette(pal);
+
+
+
     QFont fnt=font();
     if ( mw->retMiniMode() )
 #ifdef Q_WS_HILDON
@@ -57,8 +70,8 @@ SessionButton::SessionButton ( ONMainWindow* mw,QWidget *parent, QString id )
     cmdBox=new QComboBox ( this );
     cmdBox->setMouseTracking ( true );
     cmdBox->setFrame ( false );
-// 	cmdBox->setEditable ( true );
     QPalette cpal=cmdBox->palette();
+
     cpal.setColor ( QPalette::Button,QColor ( 255,255,255 ) );
     cpal.setColor ( QPalette::Base,QColor ( 255,255,255 ) );
     cpal.setColor ( QPalette::Window,QColor ( 255,255,255 ) );
@@ -316,6 +329,8 @@ void SessionButton::redraw()
                     toString();
     rootless=st->setting()->value ( sid+"/rootless",
                                     false ).toBool();
+    published=st->setting()->value ( sid+"/published",
+                                     false ).toBool();
 
 
     cmdBox->clear();
@@ -325,6 +340,7 @@ void SessionButton::redraw()
     cmdBox->addItem ( tr ( "RDP connection" ) );
     cmdBox->addItem ( tr ( "XDMCP" ) );
     cmdBox->addItem ( tr ( "Connection to local desktop" ) );
+    cmdBox->addItem ( tr ( "Published applications" ) );
 
     cmdBox->addItems ( par->transApplicationsNames() );
 
@@ -362,6 +378,12 @@ void SessionButton::redraw()
         cmdBox->setCurrentIndex ( XDMCP );
         command=tr ( "XDMCP" );
     }
+    else if (published)
+    {
+        cmdpix.load ( par->iconsPath ( "/16x16/X.png" ) );
+        command=tr ("Published applications");
+        cmdBox->setCurrentIndex (PUBLISHED);
+    }
     else
     {
         cmdpix.load ( par->iconsPath ( "/16x16/X.png" ) );
@@ -377,6 +399,7 @@ void SessionButton::redraw()
     }
 
 
+
     cmdIcon->setPixmap ( cmdpix );
     cmd->setText ( command );
 
@@ -591,6 +614,7 @@ void SessionButton::slot_cmd_change ( const QString& command )
     cmd->setText ( command );
     QPixmap pix;
     bool newRootless=rootless;
+    published=false;
     QString cmd=command;
     if ( command=="KDE" )
     {
@@ -645,12 +669,18 @@ void SessionButton::slot_cmd_change ( const QString& command )
         cmd="LXDE";
         newRootless=false;
     }
+    if (command== tr("Published applications"))
+    {
+        published=true;
+        cmd="PUBLISHED";
+    }
     bool found=false;
     cmd=par->internAppName ( cmd,&found );
     if ( found )
         newRootless=true;
     st.setting()->setValue ( sid+"/command", ( QVariant ) cmd );
     st.setting()->setValue ( sid+"/rootless", ( QVariant ) newRootless );
+    st.setting()->setValue ( sid+"/published", ( QVariant ) published );
     st.setting()->sync();
 
 }
