@@ -1,18 +1,24 @@
-/*
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
 
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+/***************************************************************************
+ *   Copyright (C) 2005-2012 by Oleksandr Shneyder   *
+ *   oleksandr.shneyder@obviously-nice.de   *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc.,                                       *
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ ***************************************************************************/
 
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
-*/
 #include "x2goclientconfig.h"
 #include "x2gologdebug.h"
 #include "sshmasterconnection.h"
@@ -29,6 +35,12 @@
 #endif
 #include <math.h>
 
+#ifndef Q_OS_WIN
+#include <arpa/inet.h>
+#include <netinet/tcp.h>
+#endif
+
+
 #include "onmainwindow.h"
 
 #undef DEBUG
@@ -40,6 +52,11 @@ static bool isLibSshInited=false;
 SshMasterConnection::SshMasterConnection ( QString host, int port, bool acceptUnknownServers, QString user,
         QString pass, QString key,bool autologin, bool krblogin, QObject* parent ) : QThread ( parent )
 {
+#if defined ( Q_OS_DARWIN )
+    // Mac OS X provides only 512KB stack space for secondary threads.
+    // As we put a 512KB buffer on the stack later on, we need a bigger stack space.
+    setStackSize (sizeof (char) * 1024 * 1024 * 2);
+#endif
     this->host=host;
     this->port=port;
     this->user=user;
@@ -56,7 +73,7 @@ SshMasterConnection::SshMasterConnection ( QString host, int port, bool acceptUn
     else
         x2goDebug<<"starting ssh connection without kerberos authentication"<<endl;
 #endif
-kerberos=false;
+    kerberos=false;
 }
 
 SshMasterConnection::SshMasterConnection ( QString host, int port, bool acceptUnknownServers, QString user,
@@ -64,7 +81,9 @@ SshMasterConnection::SshMasterConnection ( QString host, int port, bool acceptUn
         int remotePort, QString localHost, int localPort, SshProcess* creator,
         QObject* parent, ONMainWindow* mwd ) : QThread ( parent )
 {
-
+#if defined ( Q_OS_DARWIN )
+    setStackSize (sizeof (char) * 1024 * 1024 * 2);
+#endif
     this->host=host;
     this->port=port;
     this->user=user;
@@ -124,6 +143,7 @@ void SshMasterConnection::run()
 #endif
 
 //     int verbosity=SSH_LOG_PROTOCOL;
+    long timeout = 60;
 
     my_ssh_session = ssh_new();
     if ( my_ssh_session == NULL )
@@ -140,6 +160,9 @@ void SshMasterConnection::run()
     ssh_options_set ( my_ssh_session, SSH_OPTIONS_SSH_DIR, (mainWnd->getHomeDirectory()+"/ssh").toAscii());
 #endif
 //     ssh_options_set(my_ssh_session, SSH_OPTIONS_LOG_VERBOSITY, &verbosity);
+
+    ssh_options_set(my_ssh_session, SSH_OPTIONS_TIMEOUT, &timeout);
+
     if ( !sshConnect() )
     {
         QString err=ssh_get_error ( my_ssh_session );
@@ -195,6 +218,16 @@ void SshMasterConnection::run()
         return;
     }
 
+
+#ifndef Q_OS_WIN
+    const int y=1;
+#else
+    const char y=1;
+#endif
+    socket_t session_sock=ssh_get_fd(my_ssh_session);
+    setsockopt(session_sock, IPPROTO_TCP, TCP_NODELAY,&y, sizeof(int));
+
+
     if ( reverseTunnel )
     {
         if ( channel_forward_listen ( my_ssh_session, NULL, reverseTunnelRemotePort,  NULL ) !=SSH_OK )
@@ -574,6 +607,13 @@ void SshMasterConnection::channelLoop()
                 x2goDebug<<"new forward connection"<<endl;
 #endif
                 int sock=socket ( AF_INET, SOCK_STREAM,0 );
+#ifndef Q_OS_WIN
+                const int y=1;
+#else
+                const char y=1;
+#endif
+                setsockopt(sock, IPPROTO_TCP, TCP_NODELAY,&y, sizeof(int));
+
                 struct sockaddr_in address;
                 address.sin_family=AF_INET;
                 address.sin_port=htons ( reverseTunnelLocalPort );
